/* $Id$ */
/*
 *  arch/ppc{,64}/kernel/cputable.c
 *
 *  Copyright (C) 2001 Ben. Herrenschmidt (benh@kernel.crashing.org)
 *
 *  Modifications for ppc64:
 *      Copyright (C) 2003 Dave Engebretsen <engebret@us.ibm.com>
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 */
#include <prom.h>
#include <stddef.h>

struct cpu_spec {
	/* CPU is matched via (PVR & pvr_mask) == pvr_value */
	unsigned int pvr_mask;
	unsigned int pvr_value;
	const char *cpu_name;
};

static struct cpu_spec cpu_specs_64[] = {
	{			/* Power3 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00400000,
	 .cpu_name = "POWER3 (630)",
	 },
	{			/* Power3+ */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00410000,
	 .cpu_name = "POWER3 (630+)",
	 },
	{			/* Northstar */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00330000,
	 .cpu_name = "RS64-II (northstar)",
	 },
	{			/* Pulsar */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00340000,
	 .cpu_name = "RS64-III (pulsar)",
	 },
	{			/* I-star */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00360000,
	 .cpu_name = "RS64-III (icestar)",
	 },
	{			/* S-star */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00370000,
	 .cpu_name = "RS64-IV (sstar)",
	 },
	{			/* Power4 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00350000,
	 .cpu_name = "POWER4 (gp)",
	 },
	{			/* Power4+ */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00380000,
	 .cpu_name = "POWER4+ (gq)",
	 },
	{			/* PPC970 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00390000,
	 .cpu_name = "PPC970",
	 },
	{			/* PPC970FX */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x003c0000,
	 .cpu_name = "PPC970FX",
	 },
	{			/* PPC970MP */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00440000,
	 .cpu_name = "PPC970MP",
	 },
	{			/* Power5 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x003a0000,
	 .cpu_name = "POWER5 (gr)",
	 },
	{			/* Power5 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x003b0000,
	 .cpu_name = "POWER5 (gs)",
	 },
	{			/* BE DD1.x */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00700000,
	 .cpu_name = "Broadband Engine",
	 },
	{
	 .cpu_name = NULL,
	 }
};

static struct cpu_spec cpu_specs_32[] = {
	{			/* 601 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00010000,
	 .cpu_name = "601",
	 },
	{			/* 603 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00030000,
	 .cpu_name = "603",
	 },
	{			/* 603e */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00060000,
	 .cpu_name = "603e",
	 },
	{			/* 603ev */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00070000,
	 .cpu_name = "603ev",
	 },
	{			/* 604 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00040000,
	 .cpu_name = "604",
	 },
	{			/* 604e */
	 .pvr_mask = 0xfffff000,
	 .pvr_value = 0x00090000,
	 .cpu_name = "604e",
	 },
	{			/* 604r */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00090000,
	 .cpu_name = "604r",
	 },
	{			/* 604ev */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x000a0000,
	 .cpu_name = "604ev",
	 },
	{			/* 740/750 (0x4202, don't support TAU ?) */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x00084202,
	 .cpu_name = "740/750",
	 },
	{			/* 745/755 */
	 .pvr_mask = 0xfffff000,
	 .pvr_value = 0x00083000,
	 .cpu_name = "745/755",
	 },
	{			/* 750CX (80100 and 8010x?) */
	 .pvr_mask = 0xfffffff0,
	 .pvr_value = 0x00080100,
	 .cpu_name = "750CX",
	 },
	{			/* 750CX (82201 and 82202) */
	 .pvr_mask = 0xfffffff0,
	 .pvr_value = 0x00082200,
	 .cpu_name = "750CX",
	 },
	{			/* 750CXe (82214) */
	 .pvr_mask = 0xfffffff0,
	 .pvr_value = 0x00082210,
	 .cpu_name = "750CXe",
	 },
	{			/* 750FX rev 1.x */
	 .pvr_mask = 0xffffff00,
	 .pvr_value = 0x70000100,
	 .cpu_name = "750FX",
	 },
	{			/* 750FX rev 2.0 must disable HID0[DPM] */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x70000200,
	 .cpu_name = "750FX",
	 },
	{			/* 750FX (All revs except 2.0) */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x70000000,
	 .cpu_name = "750FX",
	 },
	{			/* 750GX */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x70020000,
	 .cpu_name = "750GX",
	 },
	{			/* 740/750 (L2CR bit need fixup for 740) */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00080000,
	 .cpu_name = "740/750",
	 },
	{			/* 7400 rev 1.1 ? (no TAU) */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x000c1101,
	 .cpu_name = "7400 (1.1)",
	 },
	{			/* 7400 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x000c0000,
	 .cpu_name = "7400",
	 },
	{			/* 7410 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x800c0000,
	 .cpu_name = "7410",
	 },
	{			/* 7450 2.0 - no doze/nap */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x80000200,
	 .cpu_name = "7450",
	 },
	{			/* 7450 2.1 */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x80000201,
	 .cpu_name = "7450",
	 },
	{			/* 7450 2.3 and newer */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80000000,
	 .cpu_name = "7450",
	 },
	{			/* 7455 rev 1.x */
	 .pvr_mask = 0xffffff00,
	 .pvr_value = 0x80010100,
	 .cpu_name = "7455",
	 },
	{			/* 7455 rev 2.0 */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x80010200,
	 .cpu_name = "7455",
	 },
	{			/* 7455 others */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80010000,
	 .cpu_name = "7455",
	 },
	{			/* 7447/7457 Rev 1.0 */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x80020100,
	 .cpu_name = "7447/7457",
	 },
	{			/* 7447/7457 Rev 1.1 */
	 .pvr_mask = 0xffffffff,
	 .pvr_value = 0x80020101,
	 .cpu_name = "7447/7457",
	 },
	{			/* 7447/7457 Rev 1.2 and later */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80020000,
	 .cpu_name = "7447/7457",
	 },
	{			/* 7447A */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80030000,
	 .cpu_name = "7447A",
	 },
	{			/* 82xx (8240, 8245, 8260 are all 603e cores) */
	 .pvr_mask = 0x7fff0000,
	 .pvr_value = 0x00810000,
	 .cpu_name = "82xx",
	 },
	{			/* All G2_LE (603e core, plus some) have the same pvr */
	 .pvr_mask = 0x7fff0000,
	 .pvr_value = 0x00820000,
	 .cpu_name = "G2_LE",
	 },
	{			/* e300 (a 603e core, plus some) on 83xx */
	 .pvr_mask = 0x7fff0000,
	 .pvr_value = 0x00830000,
	 .cpu_name = "e300",
	 },
	{			/* 8xx */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00500000,
	 .cpu_name = "8xx",
	 },
	{			/* 403GC */
	 .pvr_mask = 0xffffff00,
	 .pvr_value = 0x00200200,
	 .cpu_name = "403GC",
	 },
	{			/* 403GCX */
	 .pvr_mask = 0xffffff00,
	 .pvr_value = 0x00201400,
	 .cpu_name = "403GCX",
	 },
	{			/* 403G ?? */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x00200000,
	 .cpu_name = "403G ??",
	 },
	{			/* 405GP */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x40110000,
	 .cpu_name = "405GP",
	 },
	{			/* STB 03xxx */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x40130000,
	 .cpu_name = "STB03xxx",
	 },
	{			/* STB 04xxx */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x41810000,
	 .cpu_name = "STB04xxx",
	 },
	{			/* NP405L */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x41610000,
	 .cpu_name = "NP405L",
	 },
	{			/* NP4GS3 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x40B10000,
	 .cpu_name = "NP4GS3",
	 },
	{			/* NP405H */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x41410000,
	 .cpu_name = "NP405H",
	 },
	{			/* 405GPr */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x50910000,
	 .cpu_name = "405GPr",
	 },
	{			/* STBx25xx */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x51510000,
	 .cpu_name = "STBx25xx",
	 },
	{			/* 405LP */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x41F10000,
	 .cpu_name = "405LP",
	 },
	{			/* Xilinx Virtex-II Pro  */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x20010000,
	 .cpu_name = "Virtex-II Pro",
	 },
	{			/* 405EP */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x51210000,
	 .cpu_name = "405EP",
	 },
	{
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x40000850,
	 .cpu_name = "440EP Rev. A",
	 },
	{
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x400008d3,
	 .cpu_name = "440EP Rev. B",
	 },
	{			/* 440GP Rev. B */
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x40000440,
	 .cpu_name = "440GP Rev. B",
	 },
	{			/* 440GP Rev. C */
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x40000481,
	 .cpu_name = "440GP Rev. C",
	 },
	{			/* 440GX Rev. A */
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x50000850,
	 .cpu_name = "440GX Rev. A",
	 },
	{			/* 440GX Rev. B */
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x50000851,
	 .cpu_name = "440GX Rev. B",
	 },
	{			/* 440GX Rev. C */
	 .pvr_mask = 0xf0000fff,
	 .pvr_value = 0x50000892,
	 .cpu_name = "440GX Rev. C",
	 },
	{			/* e200z5 */
	 .pvr_mask = 0xfff00000,
	 .pvr_value = 0x81000000,
	 .cpu_name = "e200z5",
	 },
	{			/* e200z6 */
	 .pvr_mask = 0xfff00000,
	 .pvr_value = 0x81100000,
	 .cpu_name = "e200z6",
	 },
	{			/* e500 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80200000,
	 .cpu_name = "e500",
	 },
	{			/* e500v2 */
	 .pvr_mask = 0xffff0000,
	 .pvr_value = 0x80210000,
	 .cpu_name = "e500v2",
	 },
	{
	 .cpu_name = NULL,
	 }
};
static const char *loop_cpu_list(const struct cpu_spec *c, unsigned long cpu)
{
	do {
		if (c->pvr_value == (cpu & c->pvr_mask))
			return c->cpu_name;
		c++;
	} while (c->cpu_name);
	return NULL;
}

int identify_cpu(void)
{
	unsigned long cpu;
	const char *c;

      asm("mfspr %0, 0x11F":"=r"(cpu));

	c = loop_cpu_list(cpu_specs_64, cpu);
	if (c) {
#ifdef DEBUG
		prom_printf("64bit cpu %s\n", c);
#endif
		return 64;
	}
	c = loop_cpu_list(cpu_specs_32, cpu);
	if (c) {
#ifdef DEBUG
		prom_printf("32bit cpu %s\n", c);
#endif
		return 32;
	}
	prom_printf("unhandled PVR: %08lx\n", cpu);
	return 0;
}
